@@ -80,6 +80,32 @@ class BaseRepository(_Repository):
         check_error(err)
         return Submodule._from_c(self, csub[0])
 
+    def update_submodules(self, submodules=None, init=False, callbacks=None):
+        if submodules is None:
+            submodules = self.listall_submodules()
+
+        # prepare options
+        opts = ffi.new('git_submodule_update_options *')
+        C.git_submodule_update_init_options(
+            opts,
+            C.GIT_SUBMODULE_UPDATE_OPTIONS_VERSION)
+        if callbacks is not None:
+            callbacks._fill_fetch_options(opts.fetch_opts)
+        if init:
+            i = 1
+        else:
+            i = 0
+
+        for submodule in submodules:
+            submodule_instance = self.lookup_submodule(submodule)
+            err = C.git_submodule_update(
+                submodule_instance._subm,
+                i,
+                opts)
+            check_error(err)
+
+        return None
+
     #
     # Mapping interface
     #
