@@ -54,7 +54,7 @@ wrap_diff(git_diff *diff, Repository *repo)
 
     py_diff = PyObject_New(Diff, &DiffType);
     if (py_diff) {
-        Py_INCREF(repo);
+        Py_XINCREF(repo);
         py_diff->repo = repo;
         py_diff->diff = diff;
     }
@@ -104,7 +104,7 @@ wrap_diff_delta(const git_diff_delta *delta)
 }
 
 PyObject *
-wrap_diff_hunk(git_patch *patch, size_t idx)
+wrap_diff_hunk(Patch *patch, size_t idx)
 {
     DiffHunk *py_hunk;
     const git_diff_hunk *hunk;
@@ -112,33 +112,17 @@ wrap_diff_hunk(git_patch *patch, size_t idx)
     size_t j, lines_in_hunk;
     int err;
 
-    err = git_patch_get_hunk(&hunk, &lines_in_hunk, patch, idx);
+    err = git_patch_get_hunk(&hunk, &lines_in_hunk, patch->patch, idx);
     if (err < 0)
         return Error_set(err);
 
     py_hunk = PyObject_New(DiffHunk, &DiffHunkType);
     if (py_hunk) {
-        py_hunk->old_start = hunk->old_start;
-        py_hunk->old_lines = hunk->old_lines;
-        py_hunk->new_start = hunk->new_start;
-        py_hunk->new_lines = hunk->new_lines;
-        py_hunk->header = to_unicode_n((const char *) &hunk->header,
-                hunk->header_len, NULL, NULL);
-
-        py_hunk->lines = PyList_New(lines_in_hunk);
-        for (j = 0; j < lines_in_hunk; ++j) {
-            PyObject *py_line = NULL;
-
-            err = git_patch_get_line_in_hunk(&line, patch, idx, j);
-            if (err < 0)
-                return Error_set(err);
-
-            py_line = wrap_diff_line(line);
-            if (py_line == NULL)
-                return NULL;
-
-            PyList_SetItem(py_hunk->lines, j, py_line);
-        }
+        Py_INCREF(patch);
+        py_hunk->patch = patch;
+        py_hunk->hunk = hunk;
+        py_hunk->idx = idx;
+        py_hunk->n_lines = lines_in_hunk;
     }
 
     return (PyObject *) py_hunk;
@@ -167,19 +151,15 @@ wrap_diff_stats(git_diff *diff)
 }
 
 PyObject *
-wrap_diff_line(const git_diff_line *line)
+wrap_diff_line(const git_diff_line *line, DiffHunk *hunk)
 {
     DiffLine *py_line;
 
     py_line = PyObject_New(DiffLine, &DiffLineType);
     if (py_line) {
-        py_line->origin = line->origin;
-        py_line->old_lineno = line->old_lineno;
-        py_line->new_lineno = line->new_lineno;
-        py_line->num_lines = line->num_lines;
-        py_line->content = to_unicode_n(line->content, line->content_len,
-                NULL, NULL);
-        py_line->content_offset = line->content_offset;
+        Py_INCREF(hunk);
+        py_line->hunk = hunk;
+        py_line->line = line;
     }
 
     return (PyObject *) py_line;
@@ -300,7 +280,7 @@ PyMemberDef DiffDelta_members[] = {
     {NULL}
 };
 
-PyGetSetDef DiffDelta_getseters[] = {
+PyGetSetDef DiffDelta_getsetters[] = {
     GETTER(DiffDelta, is_binary),
     {NULL}
 };
@@ -337,7 +317,7 @@ PyTypeObject DiffDeltaType = {
     0,                                         /* tp_iternext       */
     DiffDelta_methods,                         /* tp_methods        */
     DiffDelta_members,                         /* tp_members        */
-    DiffDelta_getseters,                       /* tp_getset         */
+    DiffDelta_getsetters,                      /* tp_getset         */
     0,                                         /* tp_base           */
     0,                                         /* tp_dict           */
     0,                                         /* tp_descr_get      */
@@ -351,21 +331,67 @@ PyTypeObject DiffDeltaType = {
 static void
 DiffLine_dealloc(DiffLine *self)
 {
-    Py_CLEAR(self->content);
+    Py_CLEAR(self->hunk);
     PyObject_Del(self);
 }
 
-PyMemberDef DiffLine_members[] = {
-    MEMBER(DiffLine, origin, T_CHAR, "Type of the diff line"),
-    MEMBER(DiffLine, old_lineno, T_INT,
-           "Line number in old file or -1 for added line"),
-    MEMBER(DiffLine, new_lineno, T_INT,
-           "Line number in new file or -1 for deleted line"),
-    MEMBER(DiffLine, num_lines, T_INT,
-           "Number of newline characters in content"),
-    MEMBER(DiffLine, content_offset, T_INT,
-           "Offset in the original file to the content"),
-    MEMBER(DiffLine, content, T_OBJECT, "Content of the diff line"),
+PyDoc_STRVAR(DiffLine_origin__doc__, "Type of the diff line");
+PyObject *
+DiffLine_origin__get__(DiffLine *self)
+{
+    return PyUnicode_FromStringAndSize(&(self->line->origin), 1);
+}
+
+PyDoc_STRVAR(DiffLine_old_lineno__doc__, "Line number in old file or -1 for added line");
+PyObject *
+DiffLine_old_lineno__get__(DiffLine *self)
+{
+    return PyInt_FromLong(self->line->old_lineno);
+}
+
+PyDoc_STRVAR(DiffLine_new_lineno__doc__, "Line number in new file or -1 for deleted line");
+PyObject *
+DiffLine_new_lineno__get__(DiffLine *self)
+{
+    return PyInt_FromLong(self->line->new_lineno);
+}
+
+PyDoc_STRVAR(DiffLine_num_lines__doc__, "Number of newline characters in content");
+PyObject *
+DiffLine_num_lines__get__(DiffLine *self)
+{
+    return PyInt_FromLong(self->line->num_lines);
+}
+
+PyDoc_STRVAR(DiffLine_content_offset__doc__, "Offset in the original file to the content");
+PyObject *
+DiffLine_content_offset__get__(DiffLine *self)
+{
+    return PyInt_FromLongLong(self->line->content_offset);
+}
+
+PyDoc_STRVAR(DiffLine_content__doc__, "Content of the diff line");
+PyObject *
+DiffLine_content__get__(DiffLine *self)
+{
+    return to_unicode_n(self->line->content, self->line->content_len, NULL, NULL);
+}
+
+PyDoc_STRVAR(DiffLine_raw_content__doc__, "Content of the diff line (byte string)");
+PyObject *
+DiffLine_raw_content__get__(DiffLine *self)
+{
+    return PyBytes_FromStringAndSize(self->line->content, self->line->content_len);
+}
+
+PyGetSetDef DiffLine_getsetters[] = {
+    GETTER(DiffLine, origin),
+    GETTER(DiffLine, old_lineno),
+    GETTER(DiffLine, new_lineno),
+    GETTER(DiffLine, num_lines),
+    GETTER(DiffLine, content_offset),
+    GETTER(DiffLine, content),
+    GETTER(DiffLine, raw_content),
     {NULL}
 };
 
@@ -400,8 +426,8 @@ PyTypeObject DiffLineType = {
     0,                                         /* tp_iter           */
     0,                                         /* tp_iternext       */
     0,                                         /* tp_methods        */
-    DiffLine_members,                          /* tp_members        */
-    0,                                         /* tp_getset         */
+    0,                                         /* tp_members        */
+    DiffLine_getsetters,                       /* tp_getset         */
     0,                                         /* tp_base           */
     0,                                         /* tp_dict           */
     0,                                         /* tp_descr_get      */
@@ -422,7 +448,7 @@ diff_get_patch_byindex(git_diff *diff, size_t idx)
     if (err < 0)
         return Error_set(err);
 
-    return (PyObject*) wrap_patch(patch);
+    return (PyObject*) wrap_patch(patch, NULL, NULL);
 }
 
 PyObject *
@@ -475,6 +501,68 @@ PyTypeObject DiffIterType = {
     (iternextfunc) DiffIter_iternext,          /* tp_iternext       */
 };
 
+PyObject *
+diff_get_delta_byindex(git_diff *diff, size_t idx)
+{
+    const git_diff_delta *delta = git_diff_get_delta(diff, idx);
+    if (delta == NULL) {
+        PyErr_SetObject(PyExc_IndexError, PyInt_FromSize_t(idx));
+        return NULL;
+    }
+
+    return (PyObject*) wrap_diff_delta(delta);
+}
+
+PyObject *
+DeltasIter_iternext(DeltasIter *self)
+{
+    if (self->i < self->n)
+        return diff_get_delta_byindex(self->diff->diff, self->i++);
+
+    PyErr_SetNone(PyExc_StopIteration);
+    return NULL;
+}
+
+void
+DeltasIter_dealloc(DeltasIter *self)
+{
+    Py_CLEAR(self->diff);
+    PyObject_Del(self);
+}
+
+PyDoc_STRVAR(DeltasIter__doc__, "Deltas iterator object.");
+
+PyTypeObject DeltasIterType = {
+    PyVarObject_HEAD_INIT(NULL, 0)
+    "_pygit2.DeltasIter",                      /* tp_name           */
+    sizeof(DeltasIter),                        /* tp_basicsize      */
+    0,                                         /* tp_itemsize       */
+    (destructor)DeltasIter_dealloc,            /* tp_dealloc        */
+    0,                                         /* tp_print          */
+    0,                                         /* tp_getattr        */
+    0,                                         /* tp_setattr        */
+    0,                                         /* tp_compare        */
+    0,                                         /* tp_repr           */
+    0,                                         /* tp_as_number      */
+    0,                                         /* tp_as_sequence    */
+    0,                                         /* tp_as_mapping     */
+    0,                                         /* tp_hash           */
+    0,                                         /* tp_call           */
+    0,                                         /* tp_str            */
+    0,                                         /* tp_getattro       */
+    0,                                         /* tp_setattro       */
+    0,                                         /* tp_as_buffer      */
+    Py_TPFLAGS_DEFAULT,                        /* tp_flags          */
+    DeltasIter__doc__,                         /* tp_doc            */
+    0,                                         /* tp_traverse       */
+    0,                                         /* tp_clear          */
+    0,                                         /* tp_richcompare    */
+    0,                                         /* tp_weaklistoffset */
+    PyObject_SelfIter,                         /* tp_iter           */
+    (iternextfunc) DeltasIter_iternext,        /* tp_iternext       */
+};
+
+
 Py_ssize_t
 Diff_len(Diff *self)
 {
@@ -482,6 +570,23 @@ Diff_len(Diff *self)
     return (Py_ssize_t)git_diff_num_deltas(self->diff);
 }
 
+PyDoc_STRVAR(Diff_deltas__doc__, "Iterate over the diff deltas.");
+
+PyObject *
+Diff_deltas__get__(Diff *self)
+{
+    DeltasIter *iter;
+
+    iter = PyObject_New(DeltasIter, &DeltasIterType);
+    if (iter != NULL) {
+        Py_INCREF(self);
+        iter->diff = self;
+        iter->i = 0;
+        iter->n = git_diff_num_deltas(self->diff);
+    }
+    return (PyObject*)iter;
+}
+
 PyDoc_STRVAR(Diff_patch__doc__,
     "Patch diff string. Can be None in some cases, such as empty commits.");
 
@@ -523,22 +628,91 @@ cleanup:
 static void
 DiffHunk_dealloc(DiffHunk *self)
 {
-    Py_CLEAR(self->header);
-    Py_CLEAR(self->lines);
+    Py_CLEAR(self->patch);
     PyObject_Del(self);
 }
 
-PyMemberDef DiffHunk_members[] = {
-    MEMBER(DiffHunk, old_start, T_INT, "Old start."),
-    MEMBER(DiffHunk, old_lines, T_INT, "Old lines."),
-    MEMBER(DiffHunk, new_start, T_INT, "New start."),
-    MEMBER(DiffHunk, new_lines, T_INT, "New lines."),
-    MEMBER(DiffHunk, header, T_OBJECT, "Header."),
-    MEMBER(DiffHunk, lines, T_OBJECT, "Lines."),
+PyDoc_STRVAR(DiffHunk_old_start__doc__, "Old start.");
+
+PyObject *
+DiffHunk_old_start__get__(DiffHunk *self)
+{
+  return PyInt_FromLong(self->hunk->old_start);
+}
+
+PyDoc_STRVAR(DiffHunk_old_lines__doc__, "Old lines.");
+
+PyObject *
+DiffHunk_old_lines__get__(DiffHunk *self)
+{
+  return PyInt_FromLong(self->hunk->old_lines);
+}
+
+PyDoc_STRVAR(DiffHunk_new_start__doc__, "New start.");
+
+PyObject *
+DiffHunk_new_start__get__(DiffHunk *self)
+{
+  return PyInt_FromLong(self->hunk->new_start);
+}
+
+PyDoc_STRVAR(DiffHunk_new_lines__doc__, "New lines.");
+
+PyObject *
+DiffHunk_new_lines__get__(DiffHunk *self)
+{
+  return PyInt_FromLong(self->hunk->new_lines);
+}
+
+PyDoc_STRVAR(DiffHunk_header__doc__, "Header.");
+
+PyObject *
+DiffHunk_header__get__(DiffHunk *self)
+{
+    return to_unicode_n((const char *) &self->hunk->header,
+                        self->hunk->header_len, NULL, NULL);
+}
+
+PyDoc_STRVAR(DiffHunk_lines__doc__, "Lines.");
+
+PyObject *
+DiffHunk_lines__get__(DiffHunk *self)
+{
+    PyObject *py_lines;
+    PyObject *py_line;
+    const git_diff_line *line;
+    size_t i;
+    int err;
+
+    // TODO Replace by an iterator
+    py_lines = PyList_New(self->n_lines);
+    for (i = 0; i < self->n_lines; ++i) {
+        PyObject *py_line = NULL;
+
+        err = git_patch_get_line_in_hunk(&line, self->patch->patch, self->idx, i);
+        if (err < 0)
+            return Error_set(err);
+
+        py_line = wrap_diff_line(line, self);
+        if (py_line == NULL)
+            return NULL;
+
+        PyList_SetItem(py_lines, i, py_line);
+   }
+   return py_lines;
+}
+
+
+PyGetSetDef DiffHunk_getsetters[] = {
+    GETTER(DiffHunk, old_start),
+    GETTER(DiffHunk, old_lines),
+    GETTER(DiffHunk, new_start),
+    GETTER(DiffHunk, new_lines),
+    GETTER(DiffHunk, header),
+    GETTER(DiffHunk, lines),
     {NULL}
 };
 
-
 PyDoc_STRVAR(DiffHunk__doc__, "DiffHunk object.");
 
 PyTypeObject DiffHunkType = {
@@ -570,8 +744,8 @@ PyTypeObject DiffHunkType = {
     0,                                         /* tp_iter           */
     0,                                         /* tp_iternext       */
     0,                                         /* tp_methods        */
-    DiffHunk_members,                          /* tp_members        */
-    0,                                         /* tp_getset         */
+    0,                                         /* tp_members        */
+    DiffHunk_getsetters,                       /* tp_getset         */
     0,                                         /* tp_base           */
     0,                                         /* tp_dict           */
     0,                                         /* tp_descr_get      */
@@ -660,7 +834,7 @@ PyMethodDef DiffStats_methods[] = {
     {NULL}
 };
 
-PyGetSetDef DiffStats_getseters[] = {
+PyGetSetDef DiffStats_getsetters[] = {
     GETTER(DiffStats, insertions),
     GETTER(DiffStats, deletions),
     GETTER(DiffStats, files_changed),
@@ -672,7 +846,7 @@ PyDoc_STRVAR(DiffStats__doc__, "DiffStats object.");
 PyTypeObject DiffStatsType = {
     PyVarObject_HEAD_INIT(NULL, 0)
     "_pygit2.DiffStats",                       /* tp_name           */
-    sizeof(DiffStats),                          /* tp_basicsize      */
+    sizeof(DiffStats),                         /* tp_basicsize      */
     0,                                         /* tp_itemsize       */
     (destructor)DiffStats_dealloc,             /* tp_dealloc        */
     0,                                         /* tp_print          */
@@ -697,9 +871,9 @@ PyTypeObject DiffStatsType = {
     0,                                         /* tp_weaklistoffset */
     0,                                         /* tp_iter           */
     0,                                         /* tp_iternext       */
-    DiffStats_methods,                                         /* tp_methods        */
+    DiffStats_methods,                         /* tp_methods        */
     0,                                         /* tp_members        */
-    DiffStats_getseters,                       /* tp_getset         */
+    DiffStats_getsetters,                      /* tp_getset         */
     0,                                         /* tp_base           */
     0,                                         /* tp_dict           */
     0,                                         /* tp_descr_get      */
@@ -752,9 +926,6 @@ Diff_merge(Diff *self, PyObject *args)
     if (!PyArg_ParseTuple(args, "O!", &DiffType, &py_diff))
         return NULL;
 
-    if (py_diff->repo->repo != self->repo->repo)
-        return Error_set(GIT_ERROR);
-
     err = git_diff_merge(self->diff, py_diff->diff);
     if (err < 0)
         return Error_set(err);
@@ -814,7 +985,7 @@ Diff_getitem(Diff *self, PyObject *value)
     return diff_get_patch_byindex(self->diff, i);
 }
 
-PyDoc_STRVAR(Diff_stats__doc__, "Accumulate diff statistics for all patches");
+PyDoc_STRVAR(Diff_stats__doc__, "Accumulate diff statistics for all patches.");
 
 PyObject *
 Diff_stats__get__(Diff *self)
@@ -822,6 +993,32 @@ Diff_stats__get__(Diff *self)
     return wrap_diff_stats(self->diff);
 }
 
+PyDoc_STRVAR(Diff_parse_diff__doc__,
+    "parse_diff(git_diff: str) -> Diff\n"
+    "\n"
+    "Parses a git unified diff into a diff object without a repository");
+
+static PyObject *
+Diff_parse_diff(PyObject *self, PyObject *args)
+{
+  /* A wrapper around
+   * git_diff_from_buffer
+  */
+  git_diff *diff;
+  const char *content = NULL;
+  Py_ssize_t content_len;
+  int err;
+
+  if (!PyArg_ParseTuple(args, "s#", &content, &content_len))
+    return NULL;
+
+  err = git_diff_from_buffer(&diff, content, content_len);
+  if (err < 0)
+    return Error_set(err);
+
+  return wrap_diff(diff, NULL);
+}
+
 static void
 Diff_dealloc(Diff *self)
 {
@@ -830,7 +1027,8 @@ Diff_dealloc(Diff *self)
     PyObject_Del(self);
 }
 
-PyGetSetDef Diff_getseters[] = {
+PyGetSetDef Diff_getsetters[] = {
+    GETTER(Diff, deltas),
     GETTER(Diff, patch),
     GETTER(Diff, stats),
     {NULL}
@@ -846,9 +1044,12 @@ static PyMethodDef Diff_methods[] = {
     METHOD(Diff, merge, METH_VARARGS),
     METHOD(Diff, find_similar, METH_VARARGS | METH_KEYWORDS),
     METHOD(Diff, from_c, METH_STATIC | METH_VARARGS),
+    {"parse_diff", (PyCFunction) Diff_parse_diff,
+      METH_VARARGS | METH_STATIC, Diff_parse_diff__doc__},
     {NULL}
 };
 
+/* TODO Implement Diff.patches, deprecate Diff_iter and Diff_getitem */
 
 PyDoc_STRVAR(Diff__doc__, "Diff objects.");
 
@@ -882,7 +1083,7 @@ PyTypeObject DiffType = {
     0,                                         /* tp_iternext       */
     Diff_methods,                              /* tp_methods        */
     0,                                         /* tp_members        */
-    Diff_getseters,                            /* tp_getset         */
+    Diff_getsetters,                           /* tp_getset         */
     0,                                         /* tp_base           */
     0,                                         /* tp_dict           */
     0,                                         /* tp_descr_get      */
