@@ -43,6 +43,7 @@ extern PyTypeObject OidType;
 extern PyTypeObject ObjectType;
 extern PyTypeObject CommitType;
 extern PyTypeObject DiffType;
+extern PyTypeObject DeltasIterType;
 extern PyTypeObject DiffIterType;
 extern PyTypeObject DiffDeltaType;
 extern PyTypeObject DiffFileType;
@@ -66,12 +67,13 @@ extern PyTypeObject RemoteType;
 extern PyTypeObject RefspecType;
 extern PyTypeObject NoteType;
 extern PyTypeObject NoteIterType;
+extern PyTypeObject WorktreeType;
 
 
 PyDoc_STRVAR(discover_repository__doc__,
   "discover_repository(path[, across_fs[, ceiling_dirs]]) -> str\n"
   "\n"
-  "Look for a git repository and return its path.");
+  "Look for a git repository and return its path. If not found returns None.");
 
 PyObject *
 discover_repository(PyObject *self, PyObject *args)
@@ -88,6 +90,8 @@ discover_repository(PyObject *self, PyObject *args)
 
     memset(&repo_path, 0, sizeof(git_buf));
     err = git_repository_discover(&repo_path, path, across_fs, ceiling_dirs);
+    if (err == GIT_ENOTFOUND)
+        Py_RETURN_NONE;
     if (err < 0)
         return Error_set_str(err, path);
 
@@ -296,6 +300,12 @@ moduleinit(PyObject* m)
     ADD_CONSTANT_INT(m, GIT_REF_SYMBOLIC)
     ADD_CONSTANT_INT(m, GIT_REF_LISTALL)
 
+    /*
+     * Worktree
+     */
+    INIT_TYPE(WorktreeType, NULL, NULL)
+    ADD_TYPE(m, Worktree)
+
     /*
      * Branches
      */
@@ -313,9 +323,14 @@ moduleinit(PyObject* m)
     ADD_CONSTANT_INT(m, GIT_STATUS_INDEX_NEW)
     ADD_CONSTANT_INT(m, GIT_STATUS_INDEX_MODIFIED)
     ADD_CONSTANT_INT(m, GIT_STATUS_INDEX_DELETED)
+    ADD_CONSTANT_INT(m, GIT_STATUS_INDEX_RENAMED)
+    ADD_CONSTANT_INT(m, GIT_STATUS_INDEX_TYPECHANGE)
     ADD_CONSTANT_INT(m, GIT_STATUS_WT_NEW)
     ADD_CONSTANT_INT(m, GIT_STATUS_WT_MODIFIED)
     ADD_CONSTANT_INT(m, GIT_STATUS_WT_DELETED)
+    ADD_CONSTANT_INT(m, GIT_STATUS_WT_TYPECHANGE)
+    ADD_CONSTANT_INT(m, GIT_STATUS_WT_RENAMED)
+    ADD_CONSTANT_INT(m, GIT_STATUS_WT_UNREADABLE)
     ADD_CONSTANT_INT(m, GIT_STATUS_IGNORED) /* Flags for ignored files */
     ADD_CONSTANT_INT(m, GIT_STATUS_CONFLICTED)
     /* Different checkout strategies */
@@ -335,6 +350,7 @@ moduleinit(PyObject* m)
      * Diff
      */
     INIT_TYPE(DiffType, NULL, NULL)
+    INIT_TYPE(DeltasIterType, NULL, NULL)
     INIT_TYPE(DiffIterType, NULL, NULL)
     INIT_TYPE(DiffDeltaType, NULL, NULL)
     INIT_TYPE(DiffFileType, NULL, NULL)
