@@ -62,6 +62,9 @@ PyObject* Repository_listall_reference_objects(Repository *self,
                                                PyObject *args);
 PyObject* Repository_listall_branches(Repository *self, PyObject *args);
 PyObject* Repository_lookup_reference(Repository *self, PyObject *py_name);
+PyObject* Repository_add_worktree(Repository *self, PyObject *args);
+PyObject* Repository_lookup_worktree(Repository *self, PyObject *py_name);
+PyObject* Repository_list_worktrees(Repository *self, PyObject *args);
 
 PyObject*
 Repository_create_reference(Repository *self, PyObject *args, PyObject* kw);
