@@ -31,7 +31,7 @@
 import unittest
 import pygit2
 from pygit2 import GIT_CREDTYPE_USERPASS_PLAINTEXT
-from pygit2 import Username, UserPass, Keypair, KeypairFromAgent
+from pygit2 import Username, UserPass, Keypair, KeypairFromAgent, KeypairFromMemory
 from . import utils
 
 REMOTE_NAME = 'origin'
@@ -72,6 +72,15 @@ class CredentialCreateTest(utils.NoRepoTestCase):
         cred = KeypairFromAgent(username)
         self.assertEqual((username, None, None, None), cred.credential_tuple)
 
+    def test_ssh_from_memory(self):
+        username = "git"
+        pubkey = "public key data"
+        privkey = "private key data"
+        passphrase = "secret passphrase"
+
+        cred = KeypairFromMemory(username, pubkey, privkey, passphrase)
+        self.assertEqual((username, pubkey, privkey, passphrase), cred.credential_tuple)
+
 
 class CredentialCallback(utils.RepoTestCase):
     def test_callback(self):
@@ -105,7 +114,7 @@ class CallableCredentialTest(utils.RepoTestCase):
         credentials = UserPass("libgit2", "libgit2")
         callbacks = pygit2.RemoteCallbacks(credentials=credentials)
 
-        url = "https://bitbucket.org/libgit2/testgitrepository.git"
+        url = 'https://github.com/libgit2/TestGitRepository'
         remote = self.repo.create_remote("bb", url)
         remote.fetch(callbacks=callbacks)
 
