@@ -168,6 +168,11 @@ class RepoTestCaseForMerging(AutoRepoTestCase):
     repo_spec = 'tar', 'testrepoformerging'
 
 
+class Utf8BranchRepoTestCase(AutoRepoTestCase):
+
+    repo_spec = 'tar', 'utf8branchrepo'
+
+
 class DirtyRepoTestCase(AutoRepoTestCase):
 
     repo_spec = 'tar', 'dirtyrepo'
@@ -186,3 +191,8 @@ class SubmoduleRepoTestCase(AutoRepoTestCase):
 class BinaryFileRepoTestCase(AutoRepoTestCase):
 
     repo_spec = 'tar', 'binaryfilerepo'
+
+
+class GpgSignedRepoTestCase(AutoRepoTestCase):
+
+    repo_spec = 'tar', 'gpgsigned'
